/*  plumb - the pipe shell - dump stream (in hexadecimal or decimal format)
    Copyright (C) 2012-2013  Tibor 'Igor2' Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

    Project page: http://repo.hu/projects/plumb
    Contact: plumb (at) igor2.repo.hu
*/

#include <stdlib.h>
#include <string.h>
#include "process.h"
#include "fd.h"
#include "virt.h"

typedef struct {
	int alloced;
	char *buff;
} dump_t;

static void grow(dump_t *d, int size)
{
	if (size >= d->alloced) {
		d->alloced = size + 256;
		/* no need to realloc, don't need to keep content - can avoid a memcpy */
		if (d->buff != NULL)
			free(d->buff);
		d->buff = malloc(d->alloced);
	}
}

static const char *hd="0123456789ABCDEF";

/* 2 digit hex, space spearated */
static virt_ctrl_t dump_input_hex(proc_virt_t *proc, int local_fd, const char *data, int len)
{
	dump_t *d = proc->data;
	char *s;
	unsigned char *u;

	grow(d, len*3);
	u = (unsigned char *)data;
	for(s = d->buff; len > 0; len--,u++) {
		*s = hd[((*u) & 0xf0) >> 4];   s++;
		*s = hd[(*u) & 0x0f];          s++;
		*s = ' ';                      s++;
	}
	if (s > d->buff)
		s--;
	vp_write(proc, vstdout, d->buff, s-d->buff);
	return VC_OK;
}

/* 3 digit decimal, space spearated */
static virt_ctrl_t dump_input_dec(proc_virt_t *proc, int local_fd, const char *data, int len)
{
	dump_t *d = proc->data;
	char *s;
	unsigned char *u;

	grow(d, len*4+1);
	u = (unsigned char *)data;
	for(s = d->buff; len > 0; len--,u++)
		s += sprintf(s, "%03d ", *u);

	if (s > d->buff)
		s--;

	vp_write(proc, vstdout, d->buff, s-d->buff);
	return VC_OK;
}

void dump_free(proc_virt_t *p)
{
	dump_t *d = p->data;
	if (d->buff != NULL)
		free(d->buff);
	free(d);
}

typedef struct {
	char *name;
	virt_ctrl_t (*input)(proc_virt_t *proc, int local_fd, const char *data, int len);
} format_t;

format_t fmts[] = {
	{"hex", dump_input_hex},
	{"dec", dump_input_dec},
	{NULL, NULL}
};

int vp_dump_init(proc_virt_t *p, kvlist_t *args)
{
	format_t *f;
	char *sfmt = NULL;
	vp_args_t vargs[] = {
		{"", &sfmt},
		{"format", &sfmt},
		{NULL, NULL}
	};

	vp_stdfds_set(p, VP_REQ_QUIT_ON_UNBIND, VP_REQ_QUIT_ON_UNBIND, VP_REQ_UNBOUND, VP_REQ_UNBOUND, VP_REQ_UNBOUND, VP_REQ_UNBOUND);

	vp_args(args, vargs);

	if (sfmt == NULL)
		plumb_error("dump %s without format\n", p->h.id);

	p->input = NULL;
	for(f = fmts; f->name != NULL; f++) {
		if (strcmp(f->name, sfmt) == 0)
			p->input = f->input;
	}

	if (p->input == NULL)
		plumb_error("dump %s with unknown format\n", p->h.id);

	p->free  = dump_free;
	p->data  = calloc(1, sizeof(dump_t));

	return 0;
}
